import time
import uuid

from django_xray import traces, records, xray


class XRayMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        trace = self._create_new_segment(request)
        xray.set_current_trace(trace)

        try:
            response = self.get_response(request)
            trace.http.response_status_code = response.status_code
            return response
        finally:
            trace.end_time = time.time()
            xray.connection.send(trace)
            xray.set_current_trace(None)

    def _create_new_segment(self, request):
        trace_id = request.META.get('X-Amzn-Trace-Id')
        http_data = None
        if not trace_id:
            trace_id = '1-%08x-%s' % (int(time.time()), uuid.uuid4().hex[:24])

        http_data = records.HttpRecord(
            request_method=request.method,
            request_url=request.get_full_path(),
            request_user_agent=request.META.get('User-Agent'))

        segment = records.SegmentRecord(
            name='django.request',
            start_time=time.time(),
            end_time=None,
            trace_id=trace_id,
            http=http_data)

        return segment



import socket
import json
import threading
from django.conf import settings

tls = threading.local()
tls.trace_id = None
tls.current_trace = None


def set_current_trace(trace):
    tls.current_trace = trace


def get_current_trace():
    return getattr(tls, 'current_trace', None)


class Connection:

    def __init__(self):
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        hostname = getattr(settings, 'DJANGO_XRAY_HOST', '127.0.0.1')
        port = getattr(settings, 'DJANGO_XRAY_PORT', 2000)
        self._address = (hostname, port)

    def serialize(self, msg):
        data = '\n'.join([
            json.dumps({'format': 'json', 'version': 1}),
            json.dumps(msg.serialize())
        ])
        return data.encode('utf-8')

    def send(self, record):
        if not record.trace_id:
            return

        data = self.serialize(record)
        self._socket.sendto(data, self._address)


connection = Connection()



import uuid

import attr


@attr.s
class SegmentRecord:
    id = attr.ib(init=False, default=attr.Factory(lambda: uuid.uuid4().hex[16:]))
    name = attr.ib()
    start_time = attr.ib()
    end_time = attr.ib()
    trace_id = attr.ib()
    http = attr.ib(default=None)
    subsegments = attr.ib(default=attr.Factory(list))

    def serialize(self):
        data = {
            'name': self.name,
            'id': self.id,
            'start_time': self.start_time,
            'trace_id': self.trace_id,
            'end_time': self.end_time,
        }

        if self.http:
            data['http'] = self.http.serialize()

        if self.subsegments:
            data['subsegments'] = [
                subsegment.serialize(skip_parent_id=True)
                for subsegment in self.subsegments
            ]

        return data

    def add_subsegment(self, subsegment):
        self.subsegments.append(subsegment)


@attr.s
class SubSegmentRecord:
    id = attr.ib(init=False, default=attr.Factory(lambda: uuid.uuid4().hex[16:]))
    name = attr.ib()
    start_time = attr.ib()
    end_time = attr.ib()
    parent_id = attr.ib(default=None)
    namespace = attr.ib(default='remote')
    subsegments = attr.ib(default=attr.Factory(list))
    http = attr.ib(default=None)
    sql = attr.ib(default=None)

    def serialize(self, skip_parent_id=False):
        data = {
            'name': self.name,
            'id': self.id,
            'start_time': self.start_time,
            'end_time': self.end_time,
            'parent_id': self.parent_id,
            'namespace': self.namespace,
        }

        if self.http:
            data['http'] = self.http.serialize()

        if self.sql:
            data['sql'] = self.sql.serialize()

        if skip_parent_id:
            data['parent_id'] = None

        if self.subsegments:
            data['subsegments'] = [
                subsegment.serialize(skip_parent_id=True)
                for subsegment in self.subsegments
            ]

        return data

    def add_subsegment(self, subsegment):
        self.subsegments.append(subsegment)


@attr.s
class HttpRecord:
    request_method = attr.ib(default=None)
    request_url = attr.ib(default=None)
    request_user_agent = attr.ib(default=None)

    response_status_code = attr.ib(default=None)

    def serialize(self):
        return {
            'request': {
                'method': self.request_method,
                'url': self.request_url,
                'user_agent': self.request_user_agent,
            },
            'response': {
                'status': self.response_status_code
            }
        }



@attr.s
class SqlRecord:
    sanitized_query = attr.ib()
    database_type = attr.ib()

    def serialize(self):
        return {
            'sanitized_query': self.sanitized_query,
            'database_type': self.database_type,
        }
